/* yubikey.c --- Implementation of Yubikey token functions.
 *
 * Written by Simon Josefsson <simon@josefsson.org>.
 * Copyright (c) 2006, 2007, 2008 Yubico AB
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "yubikey.h"

#include <stdlib.h>
//#include <stdbool.h>
#include "stdafx.h"

/* Parser. */
/*
void
yubikey_parse (const uint8_t token[32],
	       const uint8_t key[16],
	       yubikey_token_t out)
{
  memset (out, 0, sizeof(*out));
  yubikey_modhex_decode ((void*)out, token, sizeof (*out));
  yubikey_aes_decrypt ((void*)out, key);
}
*/
/* ModHex */

const char qtrans[65] = "cbdefghijklnrtuv";
const char dtrans[65] = "jxe.uidchtnbpygk";
const char ctrans[65] = "cbsftdhuneikpglv";

const char* GetLayout(ModHexClass layout)
{
	switch (layout)
	{
	case ModHexClass::Dvorak:
		return dtrans;
	case ModHexClass::Colemak:
		return ctrans;
	default:
		return qtrans;
	}
}

bool yubikey_modhex_detect(const uint8_t *src, size_t srcSize, ModHexClass layout)
{
	const char *p1;
	const char* trans = GetLayout(layout);
	for(; *src && srcSize > 0; src++, srcSize--)
	{
		if ((p1 = strchr(trans, *src)) == NULL)
		{
			return false;
		}
	}
	return true;
}

// sharing spiritually with YubiAuth/ModHex.cs
ModHexClass yubikey_modhex_classify(const uint8_t *src, size_t srcSize)
{
	ModHexClass result = ModHexClass::NotModHex;
	if (src == NULL)
		result = ModHexClass::Null;
	else if (srcSize <= 0)
		result = ModHexClass::Zero;
	else
	{
		if (yubikey_modhex_detect(src, srcSize, ModHexClass::Qwerty))
			result |= ModHexClass::Qwerty;
		if (yubikey_modhex_detect(src, srcSize, ModHexClass::Dvorak))
			result |= ModHexClass::Dvorak;
		if (yubikey_modhex_detect(src, srcSize, ModHexClass::Colemak))
			result |= ModHexClass::Colemak;
	}
	return result;
}

std::list<ModHexClass> yubikey_modhex_serialize(ModHexClass layouts)
{
	std::list<ModHexClass> list;
	if ((layouts & ModHexClass::Qwerty) == ModHexClass::Qwerty)
		list.push_back(ModHexClass::Qwerty);
	if ((layouts & ModHexClass::Dvorak) == ModHexClass::Dvorak)
		list.push_back(ModHexClass::Dvorak);
	if ((layouts & ModHexClass::Colemak) == ModHexClass::Colemak)
		list.push_back(ModHexClass::Colemak);
	if (list.size() == 0)
		list.push_back(ModHexClass::NotModHex);
	return list;
}

void
yubikey_modhex_encode (uint8_t *dst, const uint8_t *src, size_t srcSize, ModHexClass layout)
{
  const char* trans = GetLayout(layout);
  while (srcSize--)
    {
      *dst++ = trans[(*src >> 4) & 0xf];
      *dst++ = trans[*src++ & 0xf];
    }

  *dst = '\0';
}

void
yubikey_modhex_decode (uint8_t *dst, const uint8_t *src, size_t dstSize, ModHexClass layout)
{
  char b;
  bool flag = false;
  const char *p1;

  const char* trans = GetLayout(layout);
  for (; *src && dstSize > 0; src++)
    {
      if ((p1 = strchr (trans, *src)) == NULL)
	b = 0;
      else
	b = (char) (p1 - trans);

      if ((flag = !flag))
	*dst = b;
      else
	{
	  *dst = (*dst << 4) | b;
	  dst++;
	  dstSize--;
	}
    }
  while (dstSize--)
    *dst++ = 0;
}

/* Crc */

uint16_t 
yubikey_crc16 (const uint8_t *buf, size_t buf_size)
{
  uint16_t m_crc = 0xffff;

  while (buf_size--)
    {
      int i, j;
      m_crc ^= (uint8_t) * buf++ & 0xFF;
      for (i = 0; i < 8; i++)
	{
	  j = m_crc & 1;
	  m_crc >>= 1;
	  if (j)
	    m_crc ^= 0x8408;
	}
    }

  return m_crc;
}
